/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/**
 * @author <a href="mailto:dev@felix.apache.org">Felix Project Team</a>
 */

// This javascript program displays the list of bundles as the ps command in the Felix shell

// TODO add options -s -l -u

function getBundleStateString(i){
    if (i == org.osgi.framework.Bundle.ACTIVE) {
        return "Active     ";
    } else if (i == org.osgi.framework.Bundle.INSTALLED) {
        return "Installed  ";
    } else if (i == org.osgi.framework.Bundle.RESOLVED) {
        return "Resolved   ";
    } else if (i == org.osgi.framework.Bundle.STARTING) {
        return "Starting   ";
    }else if (i == org.osgi.framework.Bundle.STOPPING) {
        return "Stopping   ";
    }
    return "Unknown    ";
}

var startTime = java.lang.System.currentTimeMillis();
var out=java.lang.System.out;

// get a reference to the StartLevel service
var slref = bundleContext.getServiceReference("org.osgi.service.startlevel.StartLevel");
var startLevel = null;
if (slref != null) {
    startLevel = bundleContext.getService(slref);
}
if (startLevel == null){
    out.println("StartLevel service is unavailable.");
}

// Display active start level.
if (startLevel != null){
    out.println("START LEVEL " + startLevel.getStartLevel());
}


var bundles=bundleContext.getBundles();
out.println("ID   State         Name");
for (var i = 0; i < bundles.length; i++) {
	var bdl = bundles [i];
	
	var id=""+bdl.getBundleId();
	while (id.length < 4){
       id = " " + id;
    }	
  	out.print("[" + id + "] ");

  	out.print("[" + getBundleStateString(bdl.getState()) + "] ");

	var level;
    if (startLevel == null){
        level = "1";
    } else {
        level = ""+startLevel.getBundleStartLevel(bdl);
    }
    while (level.length < 5){
        level = " " + level;
    }
  	out.print("[" + level + "] ");

    var name = bdl.getSymbolicName();
    name = (name == null) ? "<no symbolic name>" : name;
    var version = bdl.getHeaders().get(org.osgi.framework.Constants.BUNDLE_VERSION);
    name = (version != null) ? name + " (" + version + ")" : name;
  	out.print(name);
  	
  	//out.print("\n\t\t\t" + bdl.getLocation());
  	
    out.println();
}

if (startLevel != null) {
    bundleContext.ungetService(slref);
    startLevel = null; // avoid stale reference
}

var endTime = java.lang.System.currentTimeMillis();
out.println("\nScript executed in " + (endTime-startTime)/1000.0+ " sec.");
